# Generated by roxygen2: do not edit by hand

S3method(plot,greedy_experimental_design_search)
S3method(plot,greedy_multiple_kernel_experimental_design)
S3method(print,binary_match_structure)
S3method(print,binary_then_greedy_experimental_design)
S3method(print,binary_then_rerandomization_experimental_design)
S3method(print,greedy_experimental_design_search)
S3method(print,greedy_multiple_kernel_experimental_design)
S3method(print,karp_experimental_design_search)
S3method(print,optimal_experimental_design_search)
S3method(print,pairwise_matching_experimental_design_search)
S3method(print,rerandomization_experimental_design_search)
S3method(summary,binary_match_structure)
S3method(summary,binary_then_greedy_experimental_design)
S3method(summary,binary_then_rerandomization_experimental_design)
S3method(summary,greedy_experimental_design_search)
S3method(summary,greedy_multiple_kernel_experimental_design)
S3method(summary,karp_experimental_design_search)
S3method(summary,optimal_experimental_design_search)
S3method(summary,pairwise_matching_experimental_design_search)
S3method(summary,rerandomization_experimental_design_search)
export(all_elements_same_cpp_wrap)
export(complete_randomization)
export(complete_randomization_with_forced_balanced)
export(computeBinaryMatchStructure)
export(compute_distance_matrix_cpp_wrap)
export(compute_distance_matrix_gpu)
export(compute_gram_matrix)
export(compute_kernel_matrix_gpu)
export(compute_multiple_kernel_objective_vals_gpu)
export(compute_objective_val)
export(compute_objective_vals_gpu)
export(compute_randomization_metrics)
export(compute_randomization_metrics_gpu)
export(create_all_ys_cpp_wrap)
export(full_greedy_search_gpu)
export(ged_gpu_available)
export(ged_gpu_devices)
export(gen_pm_designs_cpp_wrap)
export(gen_var_cov_matrix_block_designs)
export(generate_block_design_cpp_wrap)
export(generate_stdzied_design_matrix)
export(greedy_orthogonalization_curation)
export(greedy_orthogonalization_curation2)
export(hadamardExperimentalDesign)
export(imbalanced_block_designs)
export(imbalanced_complete_randomization)
export(initBinaryMatchExperimentalDesignSearchObject)
export(initBinaryMatchFollowedByGreedyExperimentalDesignSearchObject)
export(initBinaryMatchFollowedByRerandomizationDesignSearchObject)
export(initGreedyExperimentalDesignObject)
export(initGreedyMultipleKernelExperimentalDesignObject)
export(initGurobiNumericalOptimizationExperimentalDesignObject)
export(initKarpExperimentalDesignObject)
export(initOptimalExperimentalDesignObject)
export(initRerandomizationExperimentalDesignObject)
export(optimize_asymmetric_treatment_assignment)
export(plot_obj_val_by_iter)
export(plot_obj_val_order_statistic)
export(resultsBinaryMatchSearch)
export(resultsBinaryMatchThenGreedySearch)
export(resultsBinaryMatchThenRerandomizationSearch)
export(resultsGreedySearch)
export(resultsGurobiNumericalOptimizeSearch)
export(resultsKarpSearch)
export(resultsMultipleKernelGreedySearch)
export(resultsOptimalSearch)
export(resultsRerandomizationSearch)
export(safe_cov_inverse)
export(searchTimeElapsed)
export(shuffle_cpp_wrap)
export(standardize_data_matrix)
export(startSearch)
export(stopSearch)
import(checkmate)
import(grDevices)
import(graphics)
import(rJava)
import(stats)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(checkmate,assertChoice)
importFrom(checkmate,assertClass)
importFrom(checkmate,assertCount)
importFrom(checkmate,assertLogical)
importFrom(checkmate,assertMatrix)
importFrom(checkmate,assertNumeric)
importFrom(checkmate,assertTRUE)
importFrom(checkmate,vname)
importFrom(nbpMatching,distancematrix)
importFrom(nbpMatching,nonbimatch)
importFrom(rlist,list.cbind)
importFrom(stringi,stri_replace_first_regex)
importFrom(stringr,str_detect)
