# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

is_symmetric_cpp <- function(D, tol) {
    .Call('_GreedyExperimentalDesign_is_symmetric_cpp', PACKAGE = 'GreedyExperimentalDesign', D, tol)
}

sort_unique_pairs_cpp <- function(pairs) {
    .Call('_GreedyExperimentalDesign_sort_unique_pairs_cpp', PACKAGE = 'GreedyExperimentalDesign', pairs)
}

generate_block_design_cpp <- function(B, nR, dummy_block) {
    .Call('_GreedyExperimentalDesign_generate_block_design_cpp', PACKAGE = 'GreedyExperimentalDesign', B, nR, dummy_block)
}

compute_distance_matrix_cpp <- function(X) {
    .Call('_GreedyExperimentalDesign_compute_distance_matrix_cpp', PACKAGE = 'GreedyExperimentalDesign', X)
}

shuffle_cpp <- function(w, seed) {
    .Call('_GreedyExperimentalDesign_shuffle_cpp', PACKAGE = 'GreedyExperimentalDesign', w, seed)
}

create_all_ys_cpp <- function(pCs, pTs, W, two_n, nY) {
    .Call('_GreedyExperimentalDesign_create_all_ys_cpp', PACKAGE = 'GreedyExperimentalDesign', pCs, pTs, W, two_n, nY)
}

all_elements_same_cpp <- function(vec) {
    .Call('_GreedyExperimentalDesign_all_elements_same_cpp', PACKAGE = 'GreedyExperimentalDesign', vec)
}

gen_pm_designs_cpp <- function(indicies_pairs, n, r) {
    .Call('_GreedyExperimentalDesign_gen_pm_designs_cpp', PACKAGE = 'GreedyExperimentalDesign', indicies_pairs, n, r)
}

greedy_orthogonalization_eliminate_cpp <- function(Rij, Rmin, stop_if_worse) {
    .Call('_GreedyExperimentalDesign_greedy_orthogonalization_eliminate_cpp', PACKAGE = 'GreedyExperimentalDesign', Rij, Rmin, stop_if_worse)
}

compute_indicTs_from_pairs_cpp <- function(pairs, w_diff, n) {
    .Call('_GreedyExperimentalDesign_compute_indicTs_from_pairs_cpp', PACKAGE = 'GreedyExperimentalDesign', pairs, w_diff, n)
}

compute_indicTs_from_pairs_lgl_cpp <- function(pairs, w_diff, n) {
    .Call('_GreedyExperimentalDesign_compute_indicTs_from_pairs_lgl_cpp', PACKAGE = 'GreedyExperimentalDesign', pairs, w_diff, n)
}

compute_indicTs_from_pairs_raw_cpp <- function(pairs, w_diff, n) {
    .Call('_GreedyExperimentalDesign_compute_indicTs_from_pairs_raw_cpp', PACKAGE = 'GreedyExperimentalDesign', pairs, w_diff, n)
}

compute_kernel_matrix_cpp <- function(X, kernel_type, poly_s) {
    .Call('_GreedyExperimentalDesign_compute_kernel_matrix_cpp', PACKAGE = 'GreedyExperimentalDesign', X, kernel_type, poly_s)
}

compute_kernel_quadratic_forms_cpp <- function(W, K) {
    .Call('_GreedyExperimentalDesign_compute_kernel_quadratic_forms_cpp', PACKAGE = 'GreedyExperimentalDesign', W, K)
}

compute_avg_abs_rijss_left <- function(Rij) {
    .Call('_GreedyExperimentalDesign_compute_avg_abs_rijss_left', PACKAGE = 'GreedyExperimentalDesign', Rij)
}

compute_mahalanobis_distance_matrix_cpp <- function(X, Sinv) {
    .Call('_GreedyExperimentalDesign_compute_mahalanobis_distance_matrix_cpp', PACKAGE = 'GreedyExperimentalDesign', X, Sinv)
}

standardize_data_matrix_cpp <- function(X) {
    .Call('_GreedyExperimentalDesign_standardize_data_matrix_cpp', PACKAGE = 'GreedyExperimentalDesign', X)
}

compute_objective_val_cpp <- function(X, indic_T, objective, inv_cov_X = NULL) {
    .Call('_GreedyExperimentalDesign_compute_objective_val_cpp', PACKAGE = 'GreedyExperimentalDesign', X, indic_T, objective, inv_cov_X)
}

compute_objective_vals_cpp <- function(X, indicTs, objective, inv_cov_X = NULL) {
    .Call('_GreedyExperimentalDesign_compute_objective_vals_cpp', PACKAGE = 'GreedyExperimentalDesign', X, indicTs, objective, inv_cov_X)
}

compute_pair_differences_cpp <- function(X, pairs) {
    .Call('_GreedyExperimentalDesign_compute_pair_differences_cpp', PACKAGE = 'GreedyExperimentalDesign', X, pairs)
}

complete_randomization_forced_balanced_cpp <- function(n, r, seed) {
    .Call('_GreedyExperimentalDesign_complete_randomization_forced_balanced_cpp', PACKAGE = 'GreedyExperimentalDesign', n, r, seed)
}

complete_randomization_imbalanced_cpp <- function(n, nT, r, seed) {
    .Call('_GreedyExperimentalDesign_complete_randomization_imbalanced_cpp', PACKAGE = 'GreedyExperimentalDesign', n, nT, r, seed)
}

complete_randomization_cpp <- function(n, r) {
    .Call('_GreedyExperimentalDesign_complete_randomization_cpp', PACKAGE = 'GreedyExperimentalDesign', n, r)
}

imbalanced_block_designs_cpp <- function(n_B, n_B_T, B, r, seed) {
    .Call('_GreedyExperimentalDesign_imbalanced_block_designs_cpp', PACKAGE = 'GreedyExperimentalDesign', n_B, n_B_T, B, r, seed)
}

compute_randomization_metrics_cpp <- function(designs) {
    .Call('_GreedyExperimentalDesign_compute_randomization_metrics_cpp', PACKAGE = 'GreedyExperimentalDesign', designs)
}

gen_var_cov_matrix_block_designs_cpp <- function(n, prop_T, B) {
    .Call('_GreedyExperimentalDesign_gen_var_cov_matrix_block_designs_cpp', PACKAGE = 'GreedyExperimentalDesign', n, prop_T, B)
}

