% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/karp_search.R
\name{print.karp_experimental_design_search}
\alias{print.karp_experimental_design_search}
\title{Prints a summary of a \code{karp_experimental_design_search} object}
\usage{
\method{print}{karp_experimental_design_search}(x, ...)
}
\arguments{
\item{x}{The \code{karp_experimental_design_search} object to be summarized in the console}

\item{...}{Other parameters to pass to the default print function}
}
\description{
Prints a summary of a \code{karp_experimental_design_search} object
}
\author{
Adam Kapelner
}
